@@ -3,6 +3,10 @@ import random
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
+from collections.abc import Callable
+from typing import List
+from matplotlib.pyplot import Figure
+from matplotlib.axes import Axes
 
 
 def getRandomColor():
@@ -43,6 +47,51 @@ class GraphBase:
         self.hatches = ['', '/', 'o', '\\', 'O', '|', '-', '+', 'x', '.', '*',
                         '/o', '\\|', '|*', '-\\', '+o', 'x*', 'o-', 'O|', 'O.', '*-']
         self.currentUsedHatch = {}
+        # 字体大小
+        self.fontSize = 12
+        # plotFilter
+        self.plotFilter: Callable[[float, float, str, str, str], bool] = None
+        # platMap
+        self.plotMap: Callable[[List[float], List[float], str, str, str], (List[float], List[float])] = None
+        # bindAx
+        self.ax: Axes = None
+
+    def bindAxes(self, ax: Axes):
+        """
+        绑定一个 ax 对象，用于绘图
+        :param ax:
+        :return:
+        """
+        self.ax = ax
+        return self
+
+    def setPlotFilter(self, plotFilter: Callable[[float, float, str, str, str], bool]):
+        """
+        设置 plot 函数过滤器，
+        [x_value, y_value, nodeName, nodeId, label] => bool value indicate is selected
+        :return:
+        """
+        self.plotFilter = plotFilter
+        return self
+
+    def setPlotMap(self, plotMap: Callable[[List[float], List[float], str, str, str], (List[float], List[float])]):
+        """
+        设置 plot 转换器
+        [x, y, nodeName, nodeId, label] => [new_x, new_y]
+        :param plotMap:
+        :return:
+        """
+        self.plotMap = plotMap
+        return self
+
+    def setFontSize(self, fontSize: int):
+        """
+        设置字体大小
+        :param fontSize:
+        :return:
+        """
+        self.fontSize = fontSize
+        return self
 
     def autoSelectHatch(self) -> str:
         """
@@ -77,34 +126,107 @@ class GraphBase:
             self.currentUsedColor[randomColor] = randomColor
         return randomColor
 
+    def tick_direction(self, direction: str):
+        """
+        设置坐标轴 ticker 的方向，可以取 ['in', 'out', 'inout']
+        :param direction:
+        :return:
+        """
+        if self.ax is not None:
+            pass
+            self.ax.tick_params(direction=direction)
+        else:
+            plt.rcParams['xtick.direction'] = direction
+            plt.rcParams['ytick.direction'] = direction
+        return self
+
     def xlim(self, *args, **kwargs):
-        plt.xlim(*args, **kwargs)
+        if self.ax is not None:
+            self.ax.set_xlim(*args, **kwargs)
+        else:
+            plt.xlim(*args, **kwargs)
         return self
 
     def ylim(self, *args, **kwargs):
-        plt.ylim(*args, **kwargs)
+        if self.ax is not None:
+            self.ax.set_ylim(*args, **kwargs)
+        else:
+            plt.ylim(*args, **kwargs)
         return self
 
     def xticks(self, ticks=None, labels=None, **kwargs):
-        plt.xticks(ticks, labels, **kwargs)
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            self.ax.set_xticks(ticks, labels, **kwargs)
+        else:
+            plt.xticks(ticks, labels, **kwargs)
+        return self
+
+    def yticks(self, ticks=None, labels=None, **kwargs):
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            self.ax.set_yticks(ticks, labels, **kwargs)
+        else:
+            plt.yticks(ticks, labels, **kwargs)
         return self
 
     def title(self, label, fontdict=None, loc=None, pad=None, **kwargs):
-        plt.title(label, fontdict=fontdict, loc=loc, pad=pad, **kwargs)
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            self.ax.set_title(label, fontdict=fontdict, loc=loc, pad=pad, **kwargs)
+        else:
+            plt.title(label, fontdict=fontdict, loc=loc, pad=pad, **kwargs)
         return self
 
     def xlabel(self, xlabel, fontdict=None, labelpad=None, loc=None, **kwargs):
-        plt.xlabel(xlabel, fontdict, labelpad, loc=loc, **kwargs)
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            self.ax.set_xlabel(xlabel, fontdict, labelpad, loc=loc, **kwargs)
+        else:
+            plt.xlabel(xlabel, fontdict, labelpad, loc=loc, **kwargs)
         return self
 
     def ylabel(self, ylabel, fontdict=None, labelpad=None, loc=None, **kwargs):
-        plt.ylabel(ylabel, fontdict, labelpad, loc=loc, **kwargs)
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            self.ax.set_ylabel(ylabel, fontdict, labelpad, loc=loc, **kwargs)
+        else:
+            plt.ylabel(ylabel, fontdict, labelpad, loc=loc, **kwargs)
         return self
 
-    def legend(self, *args, **kwargs):
-        plt.legend(*args, **kwargs)
+    def legend(self, markerSize: int = None, markerEdgeWidth: int = None,
+               *args, **kwargs):
+        if 'fontsize' not in kwargs:
+            kwargs['fontsize'] = self.fontSize
+        if self.ax is not None:
+            lgnd = self.ax.legend(*args, **kwargs)
+        else:
+            lgnd = plt.legend(*args, **kwargs)
+        if markerSize is not None:
+            for leg in lgnd.get_lines():
+                leg.set_markersize(markerSize)
+        if markerEdgeWidth is not None:
+            for leg in lgnd.get_lines():
+                leg.set_markeredgewidth(markerEdgeWidth)
         return self
 
+    def _do_filter(self, filter_func, x: List[float], y: List[float], *args):
+        cp_x, cp_y = x.copy(), y.copy()
+        x, y = [], []
+        for i in range(len(cp_x)):
+            if filter_func(cp_x[i], cp_y[i], *args):
+                x.append(cp_x[i])
+                y.append(cp_y[i])
+        return x, y
+
+    def _do_map(self, map_func, x: List[float], y: List[float], *args):
+        return map_func(x, y, *args)
+
     def innerPlot(self, x, y, *args,
                   color: str = "&&",
                   linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
@@ -120,17 +242,28 @@ class GraphBase:
             self.currentUsedMarker[marker] = marker
         if "marker" not in kwargs:
             kwargs["marker"] = marker
-        # if "markerfacecolor" not in kwargs:
-        #     kwargs["markerfacecolor"] = markerfacecolor
-        # if "markersize" not in kwargs:
-        #     kwargs["markersize"] = markersize
+        if "markerfacecolor" not in kwargs:
+            kwargs["markerfacecolor"] = 'none'
+        if "markersize" not in kwargs:
+            kwargs["markersize"] = markersize
         # 默认自动选择 color
         if color == "&&":
             color = self.autoSelectColor()
             self.currentUsedColor[color] = color
         if "color" not in kwargs:
             kwargs["color"] = color
-        plt.scatter(x, y, *args, **kwargs)
+
+        nodeName, nodeId, label = kwargs.pop("nodeName", ""), kwargs.pop("nodeId", ""), kwargs.get("label", "")
+        if self.plotFilter is not None:
+            x, y = self._do_filter(self.plotFilter, x, y, nodeName, nodeId, label)
+
+        if self.plotMap is not None:
+            x, y = self._do_map(self.plotMap, x, y, nodeName, nodeId, label)
+
+        if self.ax is not None:
+            self.ax.plot(x, y, *args, **kwargs)
+        else:
+            plt.plot(x, y, *args, **kwargs)
         return self
 
     def innerPlotSum(self, nodeList: [NodeItem], getNode, getX, getY, *args,
@@ -208,6 +341,12 @@ class GraphBase:
         if "label" not in kwargs:
             kwargs["label"] = "Merge"
 
+        if self.plotFilter is not None:
+            x, y = self._do_filter(self.plotFilter, x, y, nodeName, nodeId)
+
+        if self.plotMap is not None:
+            x, y = self._do_map(self.plotMap, x, y, nodeName, nodeId)
+
         self.innerPlot(xMerge, yMerge, *args,
                        color=color,
                        linewidth=linewidth,
@@ -233,6 +372,7 @@ class GraphBase:
         """
         if not os.path.exists(dir):
             os.makedirs(dir)
+        # plt.tight_layout()
         plt.savefig(f"{dir}{os.sep}{name}", *args, **kwargs)
         return self
 
