@@ -1,5 +1,5 @@
 from enum import Enum
-from typing import Optional
+from typing import Optional, List
 from .common import GraphBase, NodeItem, ExportItem, ExportBase
 import numpy as np
 import pandas as pd
@@ -119,55 +119,99 @@ class NodeAppDelay:
             return
         self.typeMap[item.Type.name].append(item)
 
-    def getX(self, delayType: DelayType = DelayType.LastDelay,
-             samplingInterval: float = 1.0, raw: bool = False):
-        """
-        获取采样时间列表
-        :param delayType:
-        :param samplingInterval:
-        :return:
-        """
-        lastCount, res = 0, []
+    def _sampling_interval(self, params: dict, start: int = 0, end: int = float("inf"), x_offset: float = 0) -> dict:
+        lastCount, res, x_v, y_v, tmp = 0, {}, 0, 0, []
+        delayType, samplingInterval, raw = \
+            params.get("delayType", DelayType.LastDelay), params.get("samplingInterval", 2), \
+            params.get("raw", -1)
+
         for i in range(len(self.typeMap[DelayType.FullDelay.name])):
             fullItem, lastItem = self.typeMap[DelayType.FullDelay.name][i], \
                                  self.typeMap[DelayType.LastDelay.name][i]
             full, last = fullItem.getValueByDelayTarget(DelayTarget.DelayMS), \
                          lastItem.getValueByDelayTarget(DelayTarget.DelayMS)
-            if full - last > 10e-6:
+            # 过滤重传的包
+            if full - last > 10e-6 or last <= start or last > end:
                 continue
             item = self.typeMap[delayType.name][i]
-            if samplingInterval > 0 and not raw:
-                if int(item.Time / samplingInterval) != lastCount:
-                    lastCount = int(item.Time / samplingInterval)
+            if samplingInterval > 0:
+                # 采样
+                if raw == 0:
+                    # 1. 采样区间内平均
+                    if int(item.Time / samplingInterval) != lastCount:
+                        lastCount = int(item.Time / samplingInterval)
+                        x_v = lastCount * samplingInterval + x_offset
+                        y_v = np.average(tmp)
+                        tmp = []
+                        if y_v > start and y_v <= end:
+                            res[x_v] = y_v
+                    else:
+                        tmp.append(last)
+                elif raw > 0:
+                    # 2. 按样本数间隔采样
+                    if i % samplingInterval == 0:
+                        x_v = item.Time
+                        y_v = last
+                        if y_v > start and y_v <= end:
+                            res[x_v] = y_v
                 else:
-                    continue
-                res.append(lastCount * samplingInterval)
+                    # 3. 按时间间隔采样
+                    if int(item.Time / samplingInterval) != lastCount:
+                        lastCount = int(item.Time / samplingInterval)
+                        x_v = lastCount * samplingInterval + x_offset
+                        y_v = last
+                        if y_v > start and y_v <= end:
+                            res[x_v] = y_v
             else:
-                if full - last < 10e-6:
-                    res.append(fullItem.Time)
-        # for item in self.typeMap[delayType.name]:
-        #     full, last = item.getValueByDelayTarget(DelayType.FullDelay),\
-        #                  item.getValueByDelayTarget(DelayType.LastDelay)
-        #     if full - last < 10e-6:
-        #         # 只提取非重传的
-        #         res.append(item.Time)
-
-        # if int(item.Time / samplingInterval) != lastCount:
-        #     lastCount = int(item.Time / samplingInterval)
-        # else:
-        #     continue
-        # res.append(lastCount * samplingInterval)
-
-        # if int(item.Time / samplingInterval) != lastCount:
-        #     lastCount = int(item.Time / samplingInterval)
-        # else:
-        #     continue
-        # res.append(lastCount * samplingInterval)
+                # 不采样，使用原始数据
+                x_v = item.Time
+                y_v = last
+                if y_v > start and y_v <= end:
+                    res[x_v] = y_v
         return res
 
-    def getY(self, delayType: DelayType = DelayType.LastDelay,
-             delayTarget: DelayTarget = DelayTarget.DelayMS,
-             samplingInterval: float = 1.0, raw: bool = False):
+    def _sampling(self, params: dict, get_x=True):
+        """
+        采样
+        :param params:
+        :return:
+        """
+        lastCount, x, y = 0, [], []
+        delayType, samplingInterval, raw, anomalyIntervalList = \
+            params.get("delayType", DelayType.LastDelay), params.get("samplingInterval", 2), \
+            params.get("raw", -1), params.get("anomalyIntervalList", [])
+
+        # 1. 首先对全局进行采样
+        result = self._sampling_interval(params)
+
+        # 2. 接着对每个异常区间采样
+        for idx, interval in enumerate(anomalyIntervalList):
+            tmp_result = self._sampling_interval(
+                params, interval[0], interval[1],
+                samplingInterval * idx + 1 / len(anomalyIntervalList)
+            )
+            result.update(tmp_result)
+
+        # 3. 按时间排序
+        keys = list(result.keys())
+        keys.sort()
+        x = keys
+        if not get_x:
+            for k in keys:
+                y.append(result[k])
+
+        return x, y
+
+    def getX(self, params: dict):
+        """
+        获取采样时间列表
+        :param delayType:
+        :return:
+        """
+        x, _ = self._sampling(params, True)
+        return x
+
+    def getY(self, params: dict):
         """
         获取延迟采样列表
         :param delayType:           延迟类型
@@ -175,45 +219,8 @@ class NodeAppDelay:
         :param samplingInterval:    采样间隔
         :return:
         """
-        lastCount, res, tmp = 0, [], []
-        for i in range(len(self.typeMap[DelayType.FullDelay.name])):
-            fullItem, lastItem = self.typeMap[DelayType.FullDelay.name][i], \
-                                 self.typeMap[DelayType.LastDelay.name][i]
-            full, last = fullItem.getValueByDelayTarget(DelayTarget.DelayMS), \
-                         lastItem.getValueByDelayTarget(DelayTarget.DelayMS)
-
-            if full - last > 10e-6:
-                continue
-            item = self.typeMap[delayType.name][i]
-            if samplingInterval > 0 and not raw:
-                # if int(item.Time / samplingInterval) != lastCount:
-                #     lastCount = int(item.Time / samplingInterval)
-                # else:
-                #     tmp.append(item.getValueByDelayTarget(delayTarget))
-                #     continue
-                # res.append(np.average(tmp))
-                # tmp = []
-
-                if int(item.Time / samplingInterval) != lastCount:
-                    res.append(item.getValueByDelayTarget(delayTarget))
-                    lastCount = int(item.Time / samplingInterval)
-                else:
-                    continue
-            else:
-                res.append(last)
-        # for item in self.typeMap[delayType.name]:
-        #     full, last = item.getValueByDelayTarget(DelayType.FullDelay), \
-        #                  item.getValueByDelayTarget(DelayType.LastDelay)
-        #     if full - last < 10e-6:
-        #         res.append(last)
-
-        # if int(item.Time / samplingInterval) != lastCount:
-        #     res.append(item.getValueByDelayTarget(delayTarget))
-        #     lastCount = int(item.Time / samplingInterval)
-        # else:
-        #     continue
-
-        return res
+        _, y = self._sampling(params, False)
+        return y
 
 
 class NodeDelay:
@@ -238,8 +245,7 @@ class NodeDelay:
             self.appMap[item.AppId] = NodeAppDelay(item.Node, item.AppId)
         self.appMap[item.AppId].appendItem(item)
 
-    def getX(self, AppId: int, delayType: DelayType = DelayType.LastDelay,
-             samplingInterval: float = 1.0, raw: bool = False):
+    def getX(self, AppId: int, params: dict):
         """
         获取某个App的采样时间列表
         :param AppId:
@@ -249,11 +255,9 @@ class NodeDelay:
         """
         if AppId not in self.appMap:
             return []
-        return self.appMap[AppId].getX(delayType, samplingInterval, raw)
+        return self.appMap[AppId].getX(params)
 
-    def getY(self, AppId: int, delayType: DelayType = DelayType.LastDelay,
-             delayTarget: DelayTarget = DelayTarget.DelayMS,
-             samplingInterval: float = 1.0, raw: bool = False):
+    def getY(self, AppId: int, params: dict):
         """
         获取某个App的延迟采样列表
         :param AppId:
@@ -264,7 +268,7 @@ class NodeDelay:
         """
         if AppId not in self.appMap:
             return []
-        return self.appMap[AppId].getY(delayType, delayTarget, samplingInterval, raw)
+        return self.appMap[AppId].getY(params)
 
 
 class Delay:
@@ -312,7 +316,19 @@ class DelayExtractBase:
         self.delayType = DelayType.LastDelay
         self.delayTarget = DelayTarget.DelayMS
         self.delay = delay
-        self.raw = False
+        self.raw = 0
+        # 异常区间
+        # 1. 在进行采样时，异常区间列表中的每一个区间单独采样；
+        self.anomalyIntervalList: List[tuple] = []
+
+    def setAnomalyInterval(self, intervals: List[tuple]):
+        """
+        设置异常采样区间
+        :param intervals:
+        :return:
+        """
+        self.anomalyIntervalList = intervals
+        return self
 
     def setSamplingInterval(self, samplingInterval: float):
         """
@@ -323,7 +339,14 @@ class DelayExtractBase:
         self.samplingInterval = samplingInterval
         return self
 
-    def setRaw(self, raw: bool):
+    def setRaw(self, raw: int):
+        """
+        raw < 0 => 按时间间隔采样
+        raw > 0 => 按样本数间隔采样
+        raw == 0 => 平均采样
+        :param raw:
+        :return:
+        """
         self.raw = raw
         return self
 
@@ -349,11 +372,10 @@ class DelayExtractBase:
         return self.delay.getByNode(nodeName)
 
     def getX(self, node: NodeDelay, nodeId):
-        return node.getX(nodeId, samplingInterval=self.samplingInterval, raw=self.raw)
+        return node.getX(nodeId, self.__dict__)
 
     def getY(self, node: NodeDelay, nodeId):
-        return node.getY(nodeId, self.delayType, self.delayTarget,
-                         self.samplingInterval, self.raw)
+        return node.getY(nodeId, self.__dict__)
 
     def getUnit(self):
         return DelayTarget.getUnit(self.delayTarget.name)
@@ -380,8 +402,8 @@ class DelayGraph(GraphBase, DelayExtractBase, ExportBase):
             return self
         if "label" not in kwargs:
             kwargs["label"] = node.Node
-        x, y = node.getX(appId, samplingInterval=self.samplingInterval), \
-               node.getY(appId, self.delayType, self.delayTarget, self.samplingInterval)
+        x, y = node.getX(appId, self.__dict__), \
+               node.getY(appId, self.__dict__)
         self.select(nodeName, appId, kwargs["label"])
         super().innerPlot(x, y, *args,
                           color=color,
