@@ -201,7 +201,7 @@ class DeliveryTimeGraph(GraphBase):
         label = self.deliveryTimeCollections[0].xlabels[0]
         for collection in self.deliveryTimeCollections:
             for i in range(len(collection.xlabels)):
-                longNames.append(f"{collection.collectionName} {collection.labels[i]}")
+                longNames.append(f"{collection.labels[i]}")
                 units.append("")
                 comments.append(collection.collectionName)
                 ys.append(collection.contentDeliveryTimeGroups[i])
